@@ -4,13 +4,9 @@ from pygments.token import Keyword
 
 
 class MyCliLexer(MySqlLexer):
-    """
-    Extends MySQL lexer to add keywords.
-    """
+    """Extends MySQL lexer to add keywords."""
 
     tokens = {
-        'root': [
-            (r'\brepair\b', Keyword),
-            inherit,
-        ],
+        'root': [(r'\brepair\b', Keyword),
+                 (r'\boffset\b', Keyword), inherit],
     }
