@@ -1,3 +1,4 @@
+from __future__ import unicode_literals
 import os
 import re
 import locale
@@ -9,10 +10,11 @@ from time import sleep
 
 import click
 import sqlparse
+from configobj import ConfigObj
 
 from . import export
 from .main import special_command, NO_QUERY, PARSED_QUERY
-from .favoritequeries import favoritequeries
+from .favoritequeries import FavoriteQueries
 from .utils import handle_cd_command
 from mycli.packages.prompt_utils import confirm_destructive_query
 
@@ -21,6 +23,7 @@ use_expanded_output = False
 PAGER_ENABLED = True
 tee_file = None
 once_file = written_to_once_file = None
+favoritequeries = FavoriteQueries(ConfigObj())
 
 @export
 def set_timing_enabled(val):
@@ -32,6 +35,12 @@ def set_pager_enabled(val):
     global PAGER_ENABLED
     PAGER_ENABLED = val
 
+
+@export
+def set_favorite_queries(config):
+    global favoritequeries
+    favoritequeries = FavoriteQueries(config)
+
 @export
 def is_pager_enabled():
     return PAGER_ENABLED
@@ -374,7 +383,7 @@ def watch_query(arg, **kwargs):
 """
     if not arg:
         yield (None, None, None, usage)
-        raise StopIteration
+        return
     seconds = 5
     clear_screen = False
     statement = None
@@ -383,7 +392,7 @@ def watch_query(arg, **kwargs):
         if not arg:
             # Oops, we parsed all the arguments without finding a statement
             yield (None, None, None, usage)
-            raise StopIteration
+            return
         (current_arg, _, arg) = arg.partition(' ')
         try:
             seconds = float(current_arg)
@@ -397,7 +406,7 @@ def watch_query(arg, **kwargs):
     destructive_prompt = confirm_destructive_query(statement)
     if destructive_prompt is False:
         click.secho("Wise choice!")
-        raise StopIteration
+        return
     elif destructive_prompt is True:
         click.secho("Your call!")
     cur = kwargs['cur']
@@ -425,6 +434,6 @@ def watch_query(arg, **kwargs):
             # This prints the Ctrl-C character in its own line, which prevents
             # to print a line with the cursor positioned behind the prompt
             click.secho("", nl=True)
-            raise StopIteration
+            return
         finally:
             set_pager_enabled(old_pager_enabled)
