@@ -6,6 +6,10 @@ from pymysql.constants import FIELD_TYPE
 from pymysql.converters import (convert_mysql_timestamp, convert_datetime,
                                 convert_timedelta, convert_date, conversions,
                                 decoders)
+try:
+    import paramiko
+except:
+    paramiko = False
 
 _logger = logging.getLogger(__name__)
 
@@ -37,7 +41,8 @@ class SQLExecute(object):
                                     order by table_name,ordinal_position'''
 
     def __init__(self, database, user, password, host, port, socket, charset,
-                 local_infile, ssl=False):
+                 local_infile, ssl, ssh_user, ssh_host, ssh_port, ssh_password,
+                 ssh_key_filename):
         self.dbname = database
         self.user = user
         self.password = password
@@ -49,10 +54,17 @@ class SQLExecute(object):
         self.ssl = ssl
         self._server_type = None
         self.connection_id = None
+        self.ssh_user = ssh_user
+        self.ssh_host = ssh_host
+        self.ssh_port = ssh_port
+        self.ssh_password = ssh_password
+        self.ssh_key_filename = ssh_key_filename
         self.connect()
 
     def connect(self, database=None, user=None, password=None, host=None,
-            port=None, socket=None, charset=None, local_infile=None, ssl=None):
+                port=None, socket=None, charset=None, local_infile=None,
+                ssl=None, ssh_host=None, ssh_port=None, ssh_user=None,
+                ssh_password=None, ssh_key_filename=None):
         db = (database or self.dbname)
         user = (user or self.user)
         password = (password or self.password)
@@ -62,7 +74,13 @@ class SQLExecute(object):
         charset = (charset or self.charset)
         local_infile = (local_infile or self.local_infile)
         ssl = (ssl or self.ssl)
-        _logger.debug('Connection DB Params: \n'
+        ssh_user = (ssh_user or self.ssh_user)
+        ssh_host = (ssh_host or self.ssh_host)
+        ssh_port = (ssh_port or self.ssh_port)
+        ssh_password = (ssh_password or self.ssh_password)
+        ssh_key_filename = (ssh_key_filename or self.ssh_key_filename)
+        _logger.debug(
+            'Connection DB Params: \n'
             '\tdatabase: %r'
             '\tuser: %r'
             '\thost: %r'
@@ -70,8 +88,15 @@ class SQLExecute(object):
             '\tsocket: %r'
             '\tcharset: %r'
             '\tlocal_infile: %r'
-            '\tssl: %r',
-            database, user, host, port, socket, charset, local_infile, ssl)
+            '\tssl: %r'
+            '\tssh_user: %r'
+            '\tssh_host: %r'
+            '\tssh_port: %r'
+            '\tssh_password: %r'
+            '\tssh_key_filename: %r',
+            db, user, host, port, socket, charset, local_infile, ssl,
+            ssh_user, ssh_host, ssh_port, ssh_password, ssh_key_filename
+        )
         conv = conversions.copy()
         conv.update({
             FIELD_TYPE.TIMESTAMP: lambda obj: (convert_mysql_timestamp(obj) or obj),
@@ -80,12 +105,34 @@ class SQLExecute(object):
             FIELD_TYPE.DATE: lambda obj: (convert_date(obj) or obj),
         })
 
-        conn = pymysql.connect(database=db, user=user, password=password,
-                host=host, port=port, unix_socket=socket,
-                use_unicode=True, charset=charset, autocommit=True,
-                client_flag=pymysql.constants.CLIENT.INTERACTIVE,
-                local_infile=local_infile,
-                conv=conv, ssl=ssl)
+        defer_connect = False
+
+        if ssh_host:
+            defer_connect = True
+
+        conn = pymysql.connect(
+            database=db, user=user, password=password, host=host, port=port,
+            unix_socket=socket, use_unicode=True, charset=charset,
+            autocommit=True, client_flag=pymysql.constants.CLIENT.INTERACTIVE,
+            local_infile=local_infile, conv=conv, ssl=ssl, program_name="mycli",
+            defer_connect=defer_connect
+        )
+
+        if ssh_host and paramiko:
+            client = paramiko.SSHClient()
+            client.load_system_host_keys()
+            client.set_missing_host_key_policy(paramiko.WarningPolicy())
+            client.connect(
+                ssh_host, ssh_port, ssh_user, ssh_password,
+                key_filename=ssh_key_filename
+            )
+            chan = client.get_transport().open_channel(
+                'direct-tcpip',
+                (host, port),
+                ('0.0.0.0', 0),
+            )
+            conn.connect(chan)
+
         if hasattr(self, 'conn'):
             self.conn.close()
         self.conn = conn
@@ -174,7 +221,7 @@ class SQLExecute(object):
                 yield row
 
     def table_columns(self):
-        """Yields column names"""
+        """Yields (table name, column name) pairs"""
         with self.conn.cursor() as cur:
             _logger.debug('Columns Query. sql: %r', self.table_columns_query)
             cur.execute(self.table_columns_query % self.dbname)
@@ -263,3 +310,7 @@ class SQLExecute(object):
         for title, cur, headers, status in res:
             self.connection_id = cur.fetchone()[0]
         _logger.debug('Current connection id: %s', self.connection_id)
+
+    def change_db(self, db):
+        self.conn.select_db(db)
+        self.dbname = db
